@@ -1,9 +1,11 @@
 from __future__ import annotations as a
 
+import collections
 import dataclasses
 import functools
 import inspect
-from operator import attrgetter
+import types
+import warnings
 from types import FunctionType, MethodType
 from typing import (
     Union,
@@ -17,13 +19,13 @@ from typing import (
     Iterable,
     Any,
     Dict,
-    Set,
     List,
+    overload,
 )
 
 import typic.constraints as c
-from typic.checks import issubclass, isfrozendataclass
-from typic.compat import lru_cache
+from typic.checks import issubclass, isfrozendataclass, isbuiltintype
+from typic.compat import lru_cache, Generic
 from typic.env import Environ, EnvironmentTypeError, EnvironmentValueError
 from typic.serde.binder import BoundArguments
 from typic.serde.common import (
@@ -34,6 +36,7 @@ from typic.serde.common import (
     SerdeProtocolsT,
     DeserializerT,
     TranslatorT,
+    FieldIteratorT,
 )
 from typic.common import (
     ORIG_SETTER_NAME,
@@ -56,7 +59,7 @@ from typic.strict import (
     StrictModeT,
 )
 from typic.ext.schema import SchemaFieldT, builder as schema_builder, ObjectSchemaField
-from typic.util import origin, cached_type_hints
+from typic.util import origin, cached_type_hints, cached_signature
 from typic.types import FrozenDict, freeze
 
 __all__ = (
@@ -102,9 +105,8 @@ __all__ = (
     "WriteOnly",
 )
 
-ObjectT = TypeVar("ObjectT")
+ObjectT = TypeVar("ObjectT", bound=object)
 SchemaGenT = Callable[[Type[ObjectT]], SchemaFieldT]
-_TO_RESOLVE: Set[Union[Type["WrappedObjectT"], Callable]] = set()
 
 
 transmute = resolver.transmute
@@ -128,28 +130,33 @@ annotations = resolver.protocols
 
 
 _T = TypeVar("_T")
+_Callable = TypeVar("_Callable", bound=Callable[..., Any])
+_Func = TypeVar("_Func", bound=types.FunctionType)
+_Type = TypeVar("_Type", bound=type)
 
 
-class TypicObjectT:
-    __serde__: SerdeProtocol
+class TypicObjectT(Generic[_T]):
+    __serde__: SerdeProtocol[Type[_T]]
     __serde_flags__: SerdeFlags
     __serde_protocols__: SerdeProtocolsT
-    __setattr_original__: Callable[["WrappedObjectT", str, Any], None]
+    __setattr_original__: Callable[[_T, str, Any], None]
     __typic_resolved__: bool
+    __iter__: FieldIteratorT[_T]
     schema: SchemaGenT
-    primitive: SerializerT
-    transmute: DeserializerT
-    translate: TranslatorT
-    validate: "c.ValidatorT"
+    primitive: SerializerT[_T]
+    transmute: DeserializerT[_T]
+    translate: TranslatorT[_T]
+    validate: c.ValidatorT[_T]
     tojson: Callable[..., str]
+    iterate: FieldIteratorT[_T]
 
 
-WrappedObjectT = Union[TypicObjectT, ObjectT]
+WrappedObjectT = Union[TypicObjectT[_T], _T]
 
 
 def wrap(
-    func: Callable[..., _T], *, delay: bool = False, strict: StrictModeT = STRICT_MODE
-) -> Callable[..., _T]:
+    func: _Callable, *, delay: bool = None, strict: StrictModeT = STRICT_MODE
+) -> _Callable:
     """Wrap a callable to automatically enforce type-coercion.
 
     Parameters
@@ -166,26 +173,22 @@ def wrap(
     :py:func:`inspect.signature`
     :py:meth:`inspect.Signature.bind`
     """
-    if not delay:
-        protocols(func)
-    else:
-        _TO_RESOLVE.add(func)
+    if isinstance(delay, bool):
+        warnings.warn(
+            "The `delay` argument is no longer required and is deprecated. "
+            "It will be removed in a future version.",
+            category=DeprecationWarning,
+        )
+    protos = protocols(func, strict=cast(bool, strict))
+    params = cached_signature(func).parameters
+    enforcer = resolver.binder.get_enforcer(parameters=params, protocols=protos)
 
     @functools.wraps(func)
-    def func_wrapper(*args, **kwargs) -> _T:
-        bound = bind(func, *args, strict=strict, **kwargs)
-        return bound.eval()
+    def func_wrapper(*args, **kwargs):
+        args, kwargs = enforcer(*args, **kwargs)
+        return func(*args, **kwargs)
 
-    return func_wrapper
-
-
-_sentinel = object()
-_origsettergetter: Callable[[Any], Callable[[str, Any], None]] = attrgetter(
-    ORIG_SETTER_NAME
-)
-_typic_attrs = frozenset(
-    (SERDE_ATTR, "primitive", "tojson", "transmute", "validate", "translate")
-)
+    return cast(_Callable, func_wrapper)
 
 
 def _bind_proto(cls, proto: SerdeProtocol):
@@ -198,6 +201,8 @@ def _bind_proto(cls, proto: SerdeProtocol):
         ("translate", proto.translate),
         ("encode", proto.encode),
         ("decode", staticmethod(proto.decode)),
+        ("iterate", proto.iterate),
+        ("__iter__", proto.iterate),
     ):
         setattr(cls, n, attr)
 
@@ -207,10 +212,10 @@ def _resolve_class(
     cls: Type[ObjectT],
     *,
     strict: StrictModeT = STRICT_MODE,
-    always: bool = True,
+    always: bool = None,
     jsonschema: bool = True,
     serde: SerdeFlags = None,
-) -> Type[WrappedObjectT]:
+) -> Type[WrappedObjectT[ObjectT]]:
     # Build the namespace for the new class
     strict = cast(bool, strict)
     protos = protocols(cls, strict=strict)
@@ -222,16 +227,28 @@ def _resolve_class(
         SERDE_FLAGS_ATTR: serde,
         TYPIC_ANNOS_NAME: protos,
     }
+    frozen = isfrozendataclass(cls)
+    always = False if frozen else always
+    if always is None:
+        warnings.warn(
+            "Keyword `always` will default to `False` in a future version. "
+            "You should update your code to either explicitly declare `always=True` "
+            "or update your code to not assume values will be coerced when set.",
+            category=UserWarning,
+            stacklevel=5,
+        )
+        always = True
     if jsonschema:
         ns["schema"] = classmethod(schema)
         schema_builder.attach(cls)
 
-    # Frozen dataclasses don't use the native setattr
-    # So we wrap the init. This should be fine,
-    # just slower :(
-    if isfrozendataclass(cls):
+    # Wrap the init if
+    #   a) this is a "frozen" dataclass
+    #   b) we only want to coerce on init.
+    # N.B.: Frozen dataclasses don't use the native setattr and can't be updated.
+    if always is False:
         ns["__init__"] = wrap(cls.__init__, strict=strict)
-    # The faster way - create a new setattr that applies the protocol for a given attr
+    # For 'always', create a new setattr that applies the protocol for a given attr
     else:
         trans = freeze({x: y.transmute for x, y in protos.items()})
 
@@ -261,44 +278,7 @@ def _resolve_class(
     _bind_proto(cls, proto)
     # Track resolution state.
     setattr(cls, "__typic_resolved__", True)
-    return cls
-
-
-def _delay_resolve_class(
-    cls_: Type[ObjectT],
-    *,
-    strict: StrictModeT = STRICT_MODE,
-    jsonschema: bool = True,
-    serde: SerdeFlags = None,
-):
-    # This class ain't resolved yet.
-    setattr(cls_, "__typic_resolved__", False)
-
-    # Create a classmethod for delayed resolution.
-    def _resolve(cls):
-        if not cls.__typic_resolved__:
-            _resolve_class(cls, strict=strict, jsonschema=jsonschema, serde=serde)
-            if cls in _TO_RESOLVE:
-                _TO_RESOLVE.remove(cls)
-        cls.__typic_resolved__ = True
-
-    setattr(cls_, "resolve", classmethod(_resolve))
-
-    # Allow users to delay until first init
-    # This adds some extra cost on init that
-    # It's not too much but something to consider
-    def init(_init):
-        @functools.wraps(_init)
-        def __delayed_init(*args, **kwargs):
-            cls_.resolve()
-            _init(*args, **kwargs)
-
-        return __delayed_init
-
-    setattr(cls_, "__init__", init(cls_.__init__))
-
-    # Add the cls to the stack of those to resolve
-    _TO_RESOLVE.add(cls_)
+    return cast(Type[WrappedObjectT[ObjectT]], cls)
 
 
 def wrap_cls(
@@ -308,7 +288,8 @@ def wrap_cls(
     strict: StrictModeT = STRICT_MODE,
     jsonschema: bool = True,
     serde: SerdeFlags = SerdeFlags(),
-) -> Type[WrappedObjectT]:
+    always: bool = None,
+) -> Type[WrappedObjectT[ObjectT]]:
     """Wrap a class to automatically enforce type-coercion on init.
 
     Notes
@@ -332,17 +313,19 @@ def wrap_cls(
         Optional settings for serialization/deserialization
     """
 
-    def cls_wrapper(cls_: Type[ObjectT]) -> Type[WrappedObjectT]:
-        setattr(cls_, "__delayed__", delay)
-        if delay:
-            _delay_resolve_class(
-                cls_, strict=strict, jsonschema=jsonschema, serde=serde
+    def cls_wrapper(cls_: Type[ObjectT]) -> Type[WrappedObjectT[ObjectT]]:
+        if isinstance(delay, bool):
+            warnings.warn(
+                "The `delay` argument is no longer required and is deprecated."
+                "It will be removed in a future version.",
+                category=DeprecationWarning,
             )
-            return cast(Type[WrappedObjectT], cls_)
+        setattr(cls_, "__delayed__", False)
+        return _resolve_class(
+            cls_, strict=strict, jsonschema=jsonschema, serde=serde, always=always
+        )
 
-        return _resolve_class(cls_, strict=strict, jsonschema=jsonschema, serde=serde)
-
-    wrapped: Type[WrappedObjectT] = cls_wrapper(klass)
+    wrapped: Type[WrappedObjectT[ObjectT]] = cls_wrapper(klass)
     return wrapped
 
 
@@ -360,7 +343,34 @@ def _get_setter(cls: Type, bases: Tuple[Type, ...] = None):
     return setter
 
 
-def typed(_cls_or_callable=None, *, delay: bool = False, strict: bool = None):
+@overload
+def typed(
+    _cls_or_callable: _Type, *, delay: bool = False, strict: bool = None
+) -> Type[WrappedObjectT[_Type]]:
+    ...
+
+
+@overload
+def typed(
+    _cls_or_callable: _Func, *, delay: bool = False, strict: bool = None
+) -> _Func:
+    ...
+
+
+@overload
+def typed(
+    *, delay: bool = False, strict: bool = None
+) -> Union[Callable[[_Type], Type[WrappedObjectT[_Type]]], Callable[[_Func], _Func]]:
+    ...
+
+
+def typed(
+    _cls_or_callable=None,
+    *,
+    delay: bool = False,
+    strict: bool = None,
+    always: bool = None,
+):
     """A convenience function which automatically selects the correct wrapper.
 
     Parameters
@@ -369,6 +379,8 @@ def typed(_cls_or_callable=None, *, delay: bool = False, strict: bool = None):
         Optionally delay annotation resolution until first call.
     strict
         Turn on "validator mode": e.g. validate incoming data rather than coerce.
+    always
+        Whether classes should always coerce values on their attributes.
 
     Returns
     -------
@@ -378,8 +390,8 @@ def typed(_cls_or_callable=None, *, delay: bool = False, strict: bool = None):
 
     def _typed(obj: Union[Callable, Type[ObjectT]]):
         if inspect.isclass(obj):
-            return wrap_cls(obj, delay=delay, strict=strict)  # type: ignore
-        elif isinstance(obj, Callable):  # type: ignore
+            return wrap_cls(obj, delay=delay, strict=strict, always=always)  # type: ignore
+        elif callable(obj):  # type: ignore
             return wrap(obj, delay=delay, strict=strict)  # type: ignore
         else:
             raise TypeError(
@@ -405,11 +417,11 @@ def resolve():
     ...
     >>> typic.resolve()
     """
-    while _TO_RESOLVE:
-        obj = _TO_RESOLVE.pop()
-        protocols(obj)
-        if inspect.isclass(obj):
-            obj.resolve()
+    warnings.warn(
+        "Delayed type resolution is handled automatically as of v2.3.0. "
+        "This function is now a no-op and will be removed in a future version.",
+        category=DeprecationWarning,
+    )
 
 
 _CONSTRAINT_TYPE_MAP = {
@@ -432,8 +444,8 @@ def _get_constraint_cls(cls: Type) -> Optional[Type[c.ConstraintsT]]:
 
 def _get_maybe_multi_constraints(
     v,
-) -> Union[c.ConstraintsT, Tuple[c.ConstraintsT, ...]]:
-    cons: Union[c.ConstraintsT, Tuple[c.ConstraintsT, ...]]
+) -> Union[c.ConstraintsProtocolT, Tuple[c.ConstraintsProtocolT, ...]]:
+    cons: Union[c.ConstraintsProtocolT, Tuple[c.ConstraintsProtocolT, ...]]
     if isinstance(v, Iterable):
         cons_gen = (c.get_constraints(x) for x in v)
         cons = tuple((x for x in cons_gen if x is not None))
@@ -575,16 +587,21 @@ def constrained(
 
             return __constrained_init
 
+        name = cls_.__name__
+        if isbuiltintype(cls_):
+            name = f"Constrained{cls_.__name__.capitalize()}"
+
         cdict.update(
             __constraints__=constraints_inst,
             __parent__=constraints_inst.type,
+            __module__=cls_.__module__,
             **(
                 {"__new__": new(cls_.__new__)}
                 if constraints_inst.type in {str, bytes, int, float}
                 else {"__init__": init(cls_.__init__)}
             ),
         )
-        cls: Type[ObjectT] = cast(Type[ObjectT], type(cls_.__name__, bases, cdict))
+        cls: Type[ObjectT] = cast(Type[ObjectT], type(name, bases, cdict))
 
         return cls
 
@@ -595,13 +612,13 @@ environ = Environ(resolver)
 
 
 def settings(
-    _klass: Type[ObjectT] = None,
+    _klass=None,
     *,
     prefix: str = "",
     case_sensitive: bool = False,
     frozen: bool = True,
     aliases: Mapping = None,
-) -> Type[ObjectT]:
+):
     """Create a typed class which fetches its defaults from env vars.
 
     The resolution order of values is `default(s) -> env value(s) -> passed value(s)`.
@@ -648,7 +665,9 @@ def settings(
 
     def settings_wrapper(_cls):
         _resolve_from_env(_cls, prefix, case_sensitive, aliases)
-        cls = wrap_cls(dataclasses.dataclass(_cls, frozen=frozen), jsonschema=False)
+        cls = wrap_cls(
+            dataclasses.dataclass(_cls, frozen=frozen), jsonschema=False, always=False
+        )
         return cls
 
     return settings_wrapper(_klass) if _klass is not None else settings_wrapper
@@ -660,27 +679,34 @@ def _resolve_from_env(
     case_sensitive: bool,
     aliases: Mapping[str, str],
 ) -> Type[ObjectT]:
-    fields = {
+    fields = cached_type_hints(cls)
+    vars = {
         (f"{prefix}{x}".lower() if not case_sensitive else f"{prefix}{x}"): (x, y)
-        for x, y in cached_type_hints(cls).items()
+        for x, y in fields.items()
     }
-    names = {*fields, *aliases}
+    attr_to_aliases = collections.defaultdict(set)
+    for alias, attr in aliases.items():
+        attr_to_aliases[attr].add(alias)
+
     sentinel = object()
-    for k in names:
-        name = aliases.get(k, k)
-        attr, typ = fields[name]
+    for name in vars:
+        attr, typ = vars[name]
+        names = attr_to_aliases[name]
         field = getattr(cls, attr, sentinel)
         if field is sentinel:
             field = dataclasses.field()
         elif not isinstance(field, dataclasses.Field):
             field = dataclasses.field(default=field)
-        if (
-            field.default != dataclasses.MISSING
-            or field.default_factory != dataclasses.MISSING
-        ):
+        if field.default_factory != dataclasses.MISSING:
             continue
-        factory = environ.register(t=typ, name=name)
-        field.default_factory = functools.partial(factory, var=k, ci=not case_sensitive)
+
+        kwargs = dict(var=name, ci=not case_sensitive)
+        if field.default != dataclasses.MISSING:
+            kwargs["default"] = field.default
+            field.default = dataclasses.MISSING
+
+        factory = environ.register(typ, *names, name=name)
+        field.default_factory = functools.partial(factory, **kwargs)
         setattr(cls, attr, field)
 
     return cls
