@@ -9,10 +9,13 @@ from .array import (
     FrozenSetConstraints,
     Array,
     ArrayConstraints,
+    DequeConstraints,
 )
 from .common import (
     ValidatorT,
+    ValidateT,
     BaseConstraints,
+    ConstraintsProtocolT,
     VT,
     MultiConstraints,
     TypeConstraints,
@@ -29,8 +32,9 @@ __all__ = (
     "BytesConstraints",
     "ConstraintValueError",
     "ConstraintSyntaxError",
-    "ConstraintsT",
+    "ConstraintsProtocolT",
     "DecimalContraints",
+    "DequeConstraints",
     "DictConstraints",
     "EnumConstraints",
     "FloatContraints",
