@@ -26,7 +26,7 @@ from typic.serde.resolver import resolver
 from typic.serde.common import SerdeProtocol, Annotation
 from typic.compat import Final, TypedDict, ForwardRef, Literal
 from typic.util import get_args, origin, get_name
-from typic.checks import istypeddict, isnamedtuple, isliteral
+from typic.checks import istypeddict, isnamedtuple, isliteral, isuniontype
 from typic.types.frozendict import FrozenDict
 
 from .field import (  # type: ignore
@@ -280,7 +280,7 @@ class SchemaBuilder:
         # `use` is the based annotation we will use for building the schema
         use = getattr(anno.origin, "__parent__", anno.origin)
         # This is a flat optional, handle it separately from the Union block.
-        use = anno.resolved if use is Union and not anno.args else use
+        use = anno.resolved if isuniontype(use) and not anno.args else use
         # If there's not a static annotation, short-circuit the rest of the checks.
         schema: SchemaFieldT
         if use in {Any, anno.EMPTY}:
@@ -292,7 +292,7 @@ class SchemaBuilder:
         # {'type': ['string', 'integer']} ==
         #   {'anyOf': [{'type': 'string'}, {'type': 'integer'}]}
         # We don't care about syntactic sugar if it's functionally the same.
-        if use is Union:
+        if isuniontype(use):
             return self._handle_union(anno=anno, ro=ro, wo=wo, name=name, parent=parent)
 
         self.__stack.add(anno)
@@ -382,14 +382,19 @@ class SchemaBuilder:
             return self._flatten_multi_definitions(definitions, field)
         return field
 
-    @staticmethod
-    def defname(obj, name: str = None) -> Optional[str]:
+    @classmethod
+    def defname(cls, obj, name: str = None) -> Optional[str]:
         """Get the definition name for an object."""
         defname = name or getattr(obj, "__name__", None)
+        if defname in cls._IGNORE_NAME:
+            defname = None
         if (obj is dict or origin(obj) is dict) and name:
             defname = name
         return inflection.camelize(defname) if defname else None
 
+    # FIXME: This isn't sustainable. Figure out a better way to ignore generics.
+    _IGNORE_NAME = {"Mapping", "MutableMapping", "Dict", "Literal"}
+
     def build_schema(self, obj: Type, *, name: str = None) -> ObjectSchemaField:
         """Build a valid JSON Schema, including nested schemas."""
         if obj in self.__cache:  # pragma: nocover
