@@ -11,9 +11,21 @@ from typing import (
     Set,
     Dict,
     Any,
+    Callable,
+    Iterator,
+    Union,
+    Iterable,
 )
 
-from typic.checks import ismappingtype, isiterabletype, isliteral
+from typic.checks import (
+    ismappingtype,
+    isiterabletype,
+    isliteral,
+    isnamedtuple,
+    isiteratortype,
+    isbuiltinsubtype,
+    istypicklass,
+)
 from typic.compat import lru_cache
 from typic.gen import Block, Keyword, ParameterKind
 from typic.util import (
@@ -22,16 +34,16 @@ from typic.util import (
     safe_get_params,
     get_unique_name,
     get_defname,
+    get_name,
 )
 
 if TYPE_CHECKING:
-    from .common import Annotation, TranslatorT, SerdeProtocol, FieldIteratorT
+    from .common import Annotation, TranslatorT, SerdeProtocol
     from .resolver import Resolver
 
 
 _itemscaller = methodcaller("items")
 _valuescaller = methodcaller("values")
-_iter = iter
 
 
 class TranslatorTypeError(TypeError):
@@ -80,18 +92,23 @@ class TranslatorFactory:
 
     @staticmethod
     def _fields_from_hints(
-        kind: ParameterKind,
-        hints: Mapping[str, Type],
+        kind: ParameterKind, hints: Mapping[str, Type], exclude: Set[str]
     ) -> Mapping[str, inspect.Parameter]:
-        return {x: inspect.Parameter(x, kind, annotation=y) for x, y in hints.items()}
+        return {
+            x: inspect.Parameter(x, kind, annotation=y)
+            for x, y in hints.items()
+            if x not in exclude
+        }
 
     @staticmethod
-    def _fields_from_attrs(kind: ParameterKind, attrs: Tuple[str, ...]):
-        return {x: inspect.Parameter(x, kind) for x in attrs}
+    def _fields_from_attrs(
+        kind: ParameterKind, attrs: Tuple[str, ...], exclude: Set[str]
+    ):
+        return {x: inspect.Parameter(x, kind) for x in attrs if x not in exclude}
 
     @lru_cache(maxsize=None)
     def get_fields(
-        self, type: Type, as_source: bool = False
+        self, type: Type, as_source: bool = False, exclude: Iterable[str] = ()
     ) -> Optional[Mapping[str, inspect.Parameter]]:
         """Get the fields for the given type.
 
@@ -102,6 +119,7 @@ class TranslatorFactory:
         making this happen.
         """
         # Try first with the signature of the target if this is the target type
+        exclude = {*exclude}
         params = safe_get_params(type)
         undefined = self.sig_is_undef(params)
         if not as_source and not undefined:
@@ -117,47 +135,67 @@ class TranslatorFactory:
         # Fetch any type hints and try to use those.
         hints = cached_type_hints(type)
         if hints:
-            return self._fields_from_hints(k, hints)
+            return self._fields_from_hints(k, hints, exclude)
         # Fallback to the target object's defined attributes
         # This will basically work for ORM models, Pydantic models...
         # Anything that defines the instance using the class body.
         attrs = cached_simple_attributes(type)
         if attrs:
-            return self._fields_from_attrs(k, attrs)
+            return self._fields_from_attrs(k, attrs, exclude)
         # Can't be done.
-        return None if undefined else params
+        return None if undefined else {f: params[f] for f in params.keys() - exclude}
 
     @lru_cache(maxsize=None)
-    def iterator(self, type: Type, values: bool = False) -> FieldIteratorT:
+    def iterator(
+        self,
+        type: Type,
+        values: bool = False,
+        relaxed: bool = False,
+        exclude: Tuple[str, ...] = (),
+    ) -> IteratorT:
         """Get an iterator function for a given type, if possible."""
+        mapping, iterable, builtin, namedtuple, typicklass = (
+            ismappingtype(type),
+            isiterabletype(type),
+            isbuiltinsubtype(type),
+            isnamedtuple(type),
+            istypicklass(type),
+        )
+        if mapping:
+            return _valuescaller if values else _itemscaller
 
-        if ismappingtype(type):
-            iter = _valuescaller if values else _itemscaller
-            return iter
+        if (iterable, namedtuple, typicklass) == (True, False, False):
+            return iter if values else enumerate
 
-        if isiterabletype(type):
-            return _iter
+        if (builtin, iterable) == (True, False):
+            raise TranslatorTypeError(
+                f"Cannot get iterator for type {type.__name__!r}."
+            ) from None
 
-        fields = self.get_fields(type, as_source=True) or {}
+        fields = self.get_fields(type, as_source=True, exclude=exclude) or {}
 
-        if fields:
-            func_name = get_defname("iterator", (type, values))
-            oname = "o"
-            ctx: dict = {}
-            with Block(ctx) as main:
-                with main.f(func_name, Block.p(oname)) as func:
+        if not fields and not relaxed:
+            raise TranslatorTypeError(
+                f"Cannot get iterator for type {type.__name__!r}, "
+                f"unable to determine fields."
+            ) from None
+
+        func_name = get_defname("iterator", (type, values))
+        oname = "o"
+        ctx: dict = {}
+        with Block(ctx) as main:
+            with main.f(func_name, Block.p(oname)) as func:
+                if fields:
                     if values:
                         for f in fields:
                             func.l(f"{Keyword.YLD} {oname}.{f}")
                     else:
                         for f in fields:
                             func.l(f"{Keyword.YLD} {f!r}, {oname}.{f}")
+                else:
+                    func.l(f"{Keyword.YLD}")
 
-            return main.compile(name=func_name, ns=ctx)
-
-        raise TranslatorTypeError(
-            f"Cannot get iterator for type {type!r}, unable to determine fields."
-        ) from None
+        return main.compile(name=func_name, ns=ctx)
 
     @staticmethod
     def _get_name(source: Type, target: Type) -> str:
@@ -181,8 +219,25 @@ class TranslatorFactory:
                 fset = f"{f}={fset}"
             yield fset
 
+    def _compile_iterable_translator(self, source: Type, target: Type) -> TranslatorT:
+        func_name = self._get_name(source, target)
+        target_name = get_name(target)
+        oname = "o"
+        ismapping = ismappingtype(target)
+        iterator = self.iterator(source, not ismapping)
+        ctx = {"iterator": iterator, target_name: target}
+        with Block(ctx) as main:
+            with main.f(func_name, Block.p(oname)) as func:
+                retval = f"iterator({oname})"
+                if not isiteratortype(target):
+                    retval = f"{target_name}({retval})"
+                func.l(f"{Keyword.RET} {retval}")
+        return main.compile(name=func_name)
+
     @lru_cache(maxsize=None)
-    def _compile_translator(self, source: Type, target: Type) -> TranslatorT:
+    def _compile_translator(
+        self, source: Type, target: Type, exclude: Tuple[str, ...] = ()
+    ) -> TranslatorT:
         if isliteral(target):
             raise TranslatorTypeError(
                 f"Cannot translate to literal type: {target!r}. "
@@ -194,6 +249,8 @@ class TranslatorFactory:
         # Get the target fields for translation.
         target_fields = self.get_fields(target)
         if target_fields is None:
+            if isiterabletype(target):
+                return self._compile_iterable_translator(source, target)
             raise TranslatorTypeError(
                 f"Cannot translate to type {target!r}. "
                 f"Unable to determine target fields."
@@ -202,7 +259,7 @@ class TranslatorFactory:
         # Ensure that the target fields are a subset of the source fields.
         # We treat the target fields as the parameters for the target,
         # so this must be true.
-        fields = self.get_fields(source, as_source=True) or {}
+        fields = self.get_fields(source, as_source=True, exclude=exclude) or {}
         fields_to_pass = {x: fields[x] for x in fields.keys() & target_fields.keys()}
         required = self.required_fields(target_fields)
         if not required.issubset(fields_to_pass.keys()):
@@ -228,6 +285,12 @@ class TranslatorFactory:
         trans = main.compile(name=func_name, ns=ctx)
         return trans
 
-    def factory(self, annotation: "Annotation", target: Type) -> TranslatorT:
+    def factory(
+        self, annotation: "Annotation", target: Type, exclude: Tuple[str, ...] = ()
+    ) -> TranslatorT:
         """Generate a translator for :py:class:`typic.Annotation` -> ``type``."""
-        return self._compile_translator(annotation.resolved, target)
+        exclude = (*(exclude or annotation.serde.flags.exclude),)
+        return self._compile_translator(annotation.resolved, target, exclude=exclude)
+
+
+IteratorT = Union[Callable[[Any], Iterator[Any]], Callable[[Any], Tuple[str, Any]]]
