@@ -1,17 +1,18 @@
 from __future__ import annotations
 
 import ast
-import bdb
 import collections
-import contextlib
+import collections.abc
 import dataclasses
 import functools
 import inspect
 import sys
 import types
+import warnings
 from datetime import date, datetime, timedelta, time
 from threading import RLock
 from types import MappingProxyType, MemberDescriptorType
+import typing
 from typing import (  # type: ignore  # ironic...
     Tuple,
     Any,
@@ -35,9 +36,10 @@ from typing import (  # type: ignore  # ironic...
 )
 
 import pendulum
+from future_typing import transform_annotation
 
 import typic.checks as checks
-from typic.compat import ForwardRef, lru_cache, SpecialForm
+from typic.compat import ForwardRef, lru_cache, KW_ONLY, get_origin
 from typic.ext import json
 
 __all__ = (
@@ -50,7 +52,6 @@ __all__ = (
     "extract",
     "fastcachedmethod",
     "filtered_repr",
-    "guard_recursion",
     "get_args",
     "get_name",
     "get_defname",
@@ -177,14 +178,15 @@ def origin(annotation: Any) -> Any:
         args = get_args(actual)
         actual = args[0] if args else actual
 
-    # Extract the highest-order origin of the annotation.
-    while hasattr(actual, "__origin__"):
-        actual = actual.__origin__
+    actual = get_origin(actual) or actual
 
     # provide defaults for generics
     if not checks.isbuiltintype(actual):
         actual = _check_generics(actual)
 
+    if inspect.isroutine(actual):
+        actual = collections.abc.Callable
+
     return actual
 
 
@@ -220,32 +222,56 @@ def get_name(obj: Union[Type, ForwardRef, Callable]) -> str:
     >>> T = TypeVar("T")
     >>> typic.get_name(Dict)
     'Dict'
+    >>> typic.get_name(Dict[str, str])
+    'Dict'
     >>> typic.get_name(Any)
     'Any'
     >>> typic.get_name(dict)
     'dict'
     """
-    if hasattr(obj, "_name") and not hasattr(obj, "__name__"):
-        return obj._name or str(obj)  # type: ignore
-    elif isinstance(obj, ForwardRef):
-        return obj.__forward_arg__
-    elif obj in {NotImplemented, None, Ellipsis}:
-        return str(obj)
-    return obj.__name__
+    strobj = get_qualname(obj)
+    return strobj.rsplit(".")[-1]
 
 
 @lru_cache(maxsize=None)
-def get_qualname(obj: Type) -> str:
-    if hasattr(obj, "_name") and not hasattr(obj, "__name__"):
-        return repr(obj)
-    elif isinstance(obj, ForwardRef):
-        return obj.__forward_arg__
-    elif obj in {NotImplemented, None, Ellipsis}:
-        return str(obj)
-    qualname = getattr(obj, "__qualname__", obj.__name__)
-    if "<locals>" in qualname:
-        return obj.__name__
-    return qualname
+def get_qualname(obj: Union[Type, ForwardRef, Callable]) -> str:
+    """Safely retrieve the qualname of either a standard object or a type annotation.
+
+    Examples
+    --------
+    >>> import typic
+    >>> from typing import Dict, Any
+    >>> T = TypeVar("T")
+    >>> typic.get_qualname(Dict)
+    'typing.Dict'
+    >>> typic.get_qualname(Dict[str, str])
+    'typing.Dict'
+    >>> typic.get_qualname(Any)
+    'typing.Any'
+    >>> typic.get_qualname(dict)
+    'dict'
+    """
+    strobj = str(obj)
+    if isinstance(obj, ForwardRef):
+        strobj = str(obj.__forward_arg__)
+    isgeneric = (
+        strobj.startswith("typing.")
+        or strobj.startswith("typing_extensions.")
+        or "[" in strobj
+    )
+    # We got a typing thing.
+    if isgeneric:
+        # If this is a subscripted generic we should clean that up.
+        return strobj.split("[")[0]
+    # Easy-ish path, use name magix
+    if hasattr(obj, "__qualname__") and obj.__qualname__:  # type: ignore
+        qualname = obj.__qualname__  # type: ignore
+        if "<locals>" in qualname:
+            return qualname.rsplit(".")[-1]
+        return qualname
+    if hasattr(obj, "__name__") and obj.__name__:  # type: ignore
+        return obj.__name__  # type: ignore
+    return strobj
 
 
 @lru_cache(maxsize=None)
@@ -429,7 +455,7 @@ cached_signature = lru_cache(maxsize=None)(signature)
 
 def _safe_get_type_hints(annotation: Union[Type, Callable]) -> Dict[str, Type[Any]]:
     raw_annotations: Dict[str, Any] = {}
-    base_globals: Dict[str, Any] = {}
+    base_globals: Dict[str, Any] = {"typing": typing}
     if isinstance(annotation, type):
         for base in reversed(annotation.__mro__):
             base_globals.update(sys.modules[base.__module__].__dict__)
@@ -438,10 +464,11 @@ def _safe_get_type_hints(annotation: Union[Type, Callable]) -> Dict[str, Type[An
         raw_annotations = getattr(annotation, "__annotations__", None) or {}
         module_name = getattr(annotation, "__module__", None)
         if module_name:
-            base_globals = sys.modules[module_name].__dict__
+            base_globals.update(sys.modules[module_name].__dict__)
     annotations = {}
     for name, value in raw_annotations.items():
         if isinstance(value, str):
+            value = transform_annotation(value)
             if sys.version_info >= (3, 7):
                 value = ForwardRef(value, is_argument=False)
             else:
@@ -451,15 +478,21 @@ def _safe_get_type_hints(annotation: Union[Type, Callable]) -> Dict[str, Type[An
         except NameError:
             # this is ok, we deal with it later.
             pass
+        except TypeError as e:
+            warnings.warn(f"Couldn't evaluate type {value!r}: {e}")
+            value = Any
         annotations[name] = value
     return annotations
 
 
 def get_type_hints(obj: Union[Type, Callable]) -> Dict[str, Type[Any]]:
     try:
-        return _get_type_hints(obj)
-    except NameError:
-        return _safe_get_type_hints(obj)
+        hints = _get_type_hints(obj)
+    except (NameError, TypeError):
+        hints = _safe_get_type_hints(obj)
+    # KW_ONLY is a special sentinel to denote kw-only params in a dataclass.
+    #  We don't want to do anything with this hint/field. It's not real.
+    return {f: t for f, t in hints.items() if t is not KW_ONLY}
 
 
 cached_type_hints = lru_cache(maxsize=None)(get_type_hints)
@@ -473,6 +506,20 @@ def cached_issubclass(st: Type, t: Union[Type, Tuple[Type, ...]]) -> bool:
 
 def simple_attributes(t: Type) -> Tuple[str, ...]:
     """Extract all public, static data-attributes for a given type."""
+    # If slots are defined, this is the best way to locate static attributes.
+    if hasattr(t, "__slots__") and t.__slots__:
+        return (
+            *(
+                f
+                for f in t.__slots__
+                if not f.startswith("_")
+                # JIC - check if this is something fancy.
+                and not isinstance(getattr(t, f, ...), _DYNAMIC_ATTRIBUTES)
+            ),
+        )
+    # Otherwise we have to guess. This is inherently faulty, as attributes aren't
+    #   always defined on a class before instantiation. The alternative is reverse
+    #   engineering the constructor... yikes.
     return (
         *(
             x
@@ -524,7 +571,7 @@ def safe_get_params(obj: Type) -> Mapping[str, inspect.Parameter]:
 VT = TypeVar("VT")
 
 
-class TypeMap(Dict[Union[Type, SpecialForm], VT]):
+class TypeMap(Dict[Type, VT]):
     """A mapping of Type -> value."""
 
     def get_by_parent(self, t: Type, default: VT = None) -> Optional[VT]:
@@ -547,53 +594,6 @@ class TypeMap(Dict[Union[Type, SpecialForm], VT]):
         return default
 
 
-class RecursionDetected(RuntimeError):
-    ...
-
-
-class RecursionDetector(bdb.Bdb):  # pragma: nocover
-    """Prevent recursion from even starting.
-
-    https://stackoverflow.com/a/36663046
-
-    Warnings
-    --------
-    While the detector is tracing, no other debug tracers (i.e., codecov!) can trace.
-    """
-
-    def do_clear(self, arg):
-        pass
-
-    def __init__(self, *args):
-        bdb.Bdb.__init__(self, *args)
-        self.stack = set()
-
-    def user_call(self, frame, argument_list):
-        code = frame.f_code
-        if code in self.stack:
-            self.stack.clear()
-            raise RecursionDetected(f"Caught recursion in: {frame}")
-        self.stack.add(code)
-
-    def user_return(self, frame, return_value):
-        if frame.f_code in self.stack:
-            self.stack.remove(frame.f_code)
-
-
-_detector = RecursionDetector()
-
-
-@contextlib.contextmanager
-def guard_recursion():  # pragma: nocover
-    curtrace = sys.gettrace()
-    _detector.set_trace()
-    try:
-        yield
-    finally:
-        _detector.stack.clear()
-        sys.settrace(curtrace)
-
-
 def slotted(
     _cls: Type = None,
     *,
@@ -772,7 +772,7 @@ def get_tag_for_types(types: Tuple[Type, ...]) -> Optional[TaggedUnion]:
     if len(types) > 1:
         root = types[0]
         root_hints = cached_type_hints(root)
-        intersection = {*root_hints}
+        intersection = {k for k in root_hints if not k.startswith("_")}
         fields_by_type = {root: root_hints}
         t: Type
         for t in types[1:]:
@@ -788,7 +788,12 @@ def get_tag_for_types(types: Tuple[Type, ...]) -> Optional[TaggedUnion]:
         while intersection and tag is None:
             f = intersection.pop()
             v = getattr(root, f, empty)
-            if v is not empty and not isinstance(v, MemberDescriptorType):
+            if (
+                v is not empty
+                and not isinstance(v, MemberDescriptorType)
+                and checks.ishashable(v)
+                and not checks.isdescriptor(v)
+            ):
                 tag = f
                 continue
             rhint = root_hints[f]
